import collections


class ConnectionMetaclass(type):

    @classmethod
    def __prepare__(cls, name, bases, **kws):
        return collections.OrderedDict()

    def __new__(cls, name, bases, attrs):
        if name == 'Connection':
            return type.__new__(cls, name, bases, attrs)

        return type.__new__(cls, name, bases, attrs)



import wpath
from torm import Model
from torm import f


class Recorder(Model):
    __config__ = 'mysql'

    id = f.Int()
    name = f.Str()
    user = f.Str()
    password = f.Str()
    level = f.Int()

    update_at = f.Timestamp()
    create_at = f.Timestamp()


def test_find_many():
    rs = Recorder.count()
    print(rs)
    # for r in rs:
    #     print(r)


test_find_many()



from functools import wraps
from flask import Flask, request, views, jsonify


def parse_field(fields={}):

    default = {
        int: [int, 'optional', 0],
        str: [str, 'optional', ''],
        float: [float, 'optional', 0],
        bool: [bool, 'optional', False]
    }
    _fields = {}
    for k in fields:
        v = fields[k]
        if type(v) in (list, tuple):
            if len(v) == 3:
                _fields[k] = list(v)
            if len(v) == 1:
                _fields[k] = default[v[0]]
            if len(v) == 2:
                _fields[k] = list(v) + default[v[0]][2]
        else:
            _fields[k] = default[v]

    return _fields


def parse_form(form, fields={}):
    form = dict(form)
    _form = {}
    _errors = []
    for k in fields:
        if fields[k][1] == 'required':
            value = form.get(k)
            if value == None:
                _errors.append(ValueError(f"{k} required"))
                continue
        else:
            value = form.get(k, fields[k][2])
        try:
            value = fields[k][0](value)
        except:
            _errors.append(
                TypeError(f"{k} require {str(fields[k][0].__name__)} type value"))
            continue
        _form[k] = value
    if len(_errors):
        raise Exception("; ".join([str(e) for e in _errors]) + ".")
    return _form


def rsp_error(msg="error", data=[], code=400):
    return jsonify({'code': code, 'msg': str(msg), 'data': data})


def rsp_success(msg="success", data=[], code=200):
    return jsonify({'code': code, 'msg': str(msg), 'data': data})


def validate(fields={}, **kws):
    fields = parse_field(fields)

    def outter(handler):
        @wraps(handler)
        def wrapper(*args, **kws):
            if request.method == "GET":
                form = request.args
            else:
                if request.form:
                    form = request.form
                else:
                    form = request.get_json()
            try:
                form = parse_form(form, fields)
            except Exception as e:
                return rsp_error(msg=e)

            return handler(*args, **kws)
        return wrapper
    return outter


def validate_doc(handler):
    @wraps(handler)
    def wrapper(*args, **kws):
        doc = handler.__doc__

        doc = doc.replace(' ', '').strip('\n').split('\n')
        _field = [line.strip(' ').split(':') for line in doc]
        field = {}
        for f in _field:
            field[f[0]] = f[1]

        return handler(*args, **kws)
    return wrapper



from torm.builder.BaseBuilder import BaseBuilder
from torm.utl.Map import Map

from bson.objectid import ObjectId

import pymongo
from combomethod import combomethod

import inspect


def get_var_name(var, _depth=3):
    # 当前命名空间
    fr = inspect.currentframe()
    for i in range(_depth):
        fr = fr.f_back  # f_back获取父空间
    vars = fr.f_locals.items()  # 获取该命名空间的所有变量
    for var_name, var_val in vars:
        if var_val is var:  # 如果该变量的和var相同，则var_name就是var的变量名
            return var_name
    return None


def decode_id(result):
    if not result:
        return result
    if "_id" in result:
        result["id"] = str(result["_id"])
        result.pop("_id")
    return result


def encode_id(arg):
    if "id" in arg:
        # 如果id是空的，就去除，由数据库生成id
        if str(arg["id"]):
            arg["_id"] = ObjectId(str(arg["id"]))
        arg.pop("id")
    return arg


def parse_args(args, kwargs, _depth=3):
    where = {}

    for arg in args:
        if type(arg) in [dict, set]:
            arg = Map(arg, _depth=_depth)
            arg = encode_id(arg)
            where.update(arg)
        elif type(arg) == str:
            key = get_var_name(arg, _depth=_depth)
            if key == "id":
                where["_id"] = ObjectId(str(arg))
            else:
                where[key] = arg

    kwargs = encode_id(kwargs)

    where.update(kwargs)
    return where


class MongoBuilder(BaseBuilder):
    operators = [
        '=', '<', '>', '<=', '>=', '<>', '!=',
        'like', 'in', 'not in', 'not between', 'exist',
        'not like', 'ilike', 'not ilike', 'between', 'mod', 'all', 'size'
    ]
    operators_map = {
        '<': '$lt',
        '<=': '$lte',
        '>': '$gt',
        '>=': '$gte',
        '!=': '$ne',
        'in': '$in',
        'not in': '$nin',
        'like': '$regex',
        'ilike': '$regex',
        'not like': '$regex',
        'not ilike': '$regex',
        'exist': '$exists',
        'mod': '$mod',
        'all': '$all',
        'size': '$size',
    }
    __select__ = {}     # 检出的字段
    __where__ = {}      # 刷选
    __offset__ = 0      # offset
    __limit__ = 0    # 检索的数据条数
    __orderby__ = []    # 排序字段
    __groupby__ = []    # 排序字段
    __lock__ = None     # lock
    __join__ = []       # leftjoin
    __union__ = []      # union & unionall
    __on__ = []         # leftjoin
    __having__ = None       # having
    __subquery__ = []       # subquery
    __groupbykey__ = None   # mongo groupby 字段名

    def __init__(self, *args, **kwargs):
        super().__init__()

    @combomethod
    def reset(self):
        self.__select__ = {}
        self.__where__ = {}
        self.__offset__ = 0
        self.__limit__ = 0
        self.__orderby__ = []
        self.__groupby__ = []

        self.__lock__ = None
        self.__join__ = []
        self.__union__ = []
        self.__on__ = []

        self.__having__ = None
        self.__subquery__ = []
        self.__groupbykey__ = None

    # 增

    def create(self):
        data = self.to_ordict()
        if data:
            if isinstance(data, dict):
                data = [data]
            # data = self._set_create_time(data)
                return str(self.connection.create(self, data))
        return None

    # 删

    def delete(self):
        # if self.isinstance:
        #     self.__where__.update(self.dict())
        r = self.connection.delete(self)
        # self.reset()
        return r

    # 改

    @combomethod
    def update(self, data, **kwargs):
        r = None
        if data and isinstance(data, dict):
            # data = self._set_update_time(data)
            r = self.connection.update(self, {'$set': data}, **kwargs)
        self.reset()
        return r

    # 查

    @combomethod
    def first(self):
        self.__limit__ = 1
        data = self.get()
        if data:
            return data.pop()

        self.reset()
        return data

    @combomethod
    def get(self):
        d = self.connection.get(self)
        self.reset()
        return [Map(index) for index in d]

    # 计数

    @combomethod
    def count(self):
        return self.connection.count(self)

    # 聚合
    @combomethod
    def groupby(self, groupkey):
        groupby = {
            "$group": {}
        }
        if self.__select__.__len__() == 0:
            raise Exception('invalid select filed')
        for key, value in self.__select__.items():
            if value == 1:
                groupby['$group'].update(self.format_group_sql(key))
        self.__select__ = {}
        self.__groupby__ = groupby
        self.__groupbykey__ = groupkey
        return self

    # 常见sql操作
    @combomethod
    def select(self, *args):
        self.__select__.update({'_id': 0})
        [self.__select__.update({index: 1}) for index in args]
        return self

    @combomethod
    def where(self, *args, **kwargs):
        length = args.__len__()
        if length == 0:
            where = parse_args(args, kwargs)
            self.__where__.update(where)
        elif length == 1 and isinstance(args[0], dict):
            if args[0]:
                where = parse_args(args, kwargs)
                self.__where__.update(where)

        elif length == 2:
            if self.__where__.get('$and', None) is None:
                self.__where__['$and'] = []
            where = parse_args((), {args[0]: args[1]})
            self.__where__['$and'].append(where)

        elif length == 3:
            if self.__where__.get('$and', None) is None:
                self.__where__['$and'] = []
            if args[1] in self.operators:
                if args[1] == '=':
                    where = parse_args((), {args[0]: args[2]})
                    self.__where__['$and'].append(where)
                else:
                    where = self._compile_tuple((args[0], args[1], args[2]))
                    where = parse_args((), where)
                    self.__where__['$and'].append(where)
            else:
                raise Exception(
                    'operator key world not found: "{}"'.format(args[1]))
        else:
            raise Exception('bad parameters in where function')

        return self

    # @combomethod
    # def _check_columns_value(self, value):
    #     if self.__subquery__ and len(self.__subquery__) >= 2 and isinstance(value, str):
    #         tmp = value.split('.')
    #         if len(tmp) == 2 and tmp[0] in self._get_subquery_alias():
    #             return Expression(value)
    #     return value

    @combomethod
    def orderby(self, column, direction='asc'):
        if direction.lower() == 'asc':
            self.__orderby__.append((column, pymongo.ASCENDING))
        else:
            self.__orderby__.append((column, pymongo.DESCENDING))
        return self

    @combomethod
    def offset(self, number):
        if number < 0:
            raise Exception('offset number invalid')
        self.__offset__ = int(number)
        return self

    @combomethod
    def limit(self, number):
        if number <= 0:
            raise Exception('limit number invalid')
        self.__limit__ = int(number)
        return self

    # 执行事物
    @combomethod
    def execute(self, sql):
        return self.connection.execute(sql)
    # 将操作编译成字典

    def _compile_tuple(self, data):
        # data=('time', 'between', ['14:08:38', '14:11:37'])
        # 对应：字段，操作符，操作参数

        if data[1] == 'not like':
            return {data[0]: {self.operators_map[data[1]]: '^((?!{}).)*$'.format(data[2])}}
        elif data[1] == 'ilike':
            return {data[0]: {self.operators_map[data[1]]: data[2], '$options': 'i'}}
        elif data[1] == 'not ilike':
            return {data[0]: {self.operators_map[data[1]]: '^((?!{}).)*$'.format(data[2]), '$options': 'i'}}
        elif data[1] == 'between':
            if len(data[2]) != 2:
                raise Exception('between param error')
            return {data[0]: {'$lte': data[2][1], '$gte': data[2][0]}}
        elif data[1] == 'not between':
            if len(data[2]) != 2:
                raise Exception('not between param error')
            return {'$or': [{data[0]: {'$gt': data[2][1]}}, {data[0]: {'$lt': data[2][0]}}]}
        else:
            return {data[0]: {self.operators_map[data[1]]: data[2]}}

    @combomethod
    def InsertOne(self, item):
        self.validate_type(item)
        item = encode_id(dict(item))
        table = self.connection.table(self)
        r = table.insert_one(item)
        if r:
            return str(r.inserted_id)
        return None

    @combomethod
    def InsertMany(self, items):
        all_validate_type = all([self.validate_type(item) for item in items])
        if not all_validate_type:
            raise TypeError(f'all list element must be {self.__class__} type')

        table = self.connection.table(self)
        items = [encode_id(item.to_ordict()) for item in items]
        return table.insert_many(items)

    @combomethod
    def FindOne(self, *args, **kwargs):
        where = parse_args(args, kwargs, _depth=4)

        table = self.connection.table(self)

        item = table.find_one(where)
        if not item:
            return None

        # 将字典转成Model
        item = Map(decode_id(item))
        return item

    @combomethod
    def FindMany(self, *args, **kwargs):
        where = parse_args(args, kwargs)
        table = self.connection.table(self)
        items = table.find(where)
        if not items:
            return []
        items = [Map(decode_id(item)) for item in items]
        return items

    @combomethod
    def UpdateOne(self, where={}, item={}):

        if not where:
            return None
        where = Map(where, _depth=4)
        where = encode_id(where)

        if not item:
            return None

        if isinstance(item, self.__class__):
            item = item.to_dict()

        # 禁止更新id,_id字段
        if "id" in item:
            item.pop("id")
        if "_id" in item:
            item.pop("_id")

        table = self.connection.table(self)
        r = table.update_one(where, {"$set": item})
        return r

    @combomethod
    def DeleteOne(self, *args, **kwargs):
        where = parse_args(args, kwargs, _depth=4)
        table = self.connection.table(self)
        r = table.delete_one(where)
        return r



from configparser import ConfigParser

import os

from environs import Env


class Config(object):
    __instance = None

    # 单例模式
    def __new__(cls, *args, **kws):
        if not cls.__instance:
            cls.__instance = super().__new__(cls, *args, **kws)
        return cls.__instance

    def __init__(self):
        env = Env()
        # env.read_env()
        self.__env = env

    def reset(self, name='default'):
        path = '.env' if name == 'default' else '.env.' + name
        env = Env()
        env.read_env(path=path, override=True)
        self.__env = env

    # 读取配置文件
    def env(self, name='default'):
        path = '.env' if name == 'default' else '.env.' + name
        env = Env()

        err = None
        try:
            env.read_env(path=path, override=True)
        except Exception as e:
            if name == 'default':
                os.environ["TORM_DB_TYPE"] = "mongo"
                os.environ["TORM_DB"] = "test"
                os.environ["TORM_HOST"] = "127.0.0.1"
                os.environ["TORM_PORT"] = "27017"
                os.environ["TORM_CHARSET"] = "utf8mb4"
                env = Env()
            elif str(e) == 'Starting path not found':
                err = Exception(
                    f"env file '{path}' for config name '{name}' not found.")
            else:
                raise(e)
        if err:
            raise err

        return env

    def __getattr__(self, name):
        return self.__env(name)


config = Config()



import re


def to_snake_name(hunp_str):
    ''' 驼峰形式字符串转成下划线形式 :param hunp_str: 驼峰形式字符串 :return: 字母全小写的下划线形式字符串 '''
    # 匹配正则，匹配小写字母和大写字母的分界位置
    p = re.compile(r'([a-z]|\d)([A-Z])')
    # 这里第二个参数使用了正则分组的后向引用
    sub = re.sub(p, r'\1_\2', hunp_str).lower()
    return sub


def to_up(x, all=True):
    s = x.group()
    if len(s) == 2:
        return x.group(1)[1].upper()
    elif len(s) == 1 and all:
        return x.group(0).upper()
    return x


def to_up(x, all=True):
    s = x.group()
    if len(s) == 2:
        return x.group(1)[1].upper()
    elif len(s) == 1 and all:
        return x.group(0).upper()
    return x


def to_camel_name(underline_str):
    ''' 下划线形式字符串转成大驼峰形式 :param underline_str: 下划线形式字符串 :return: 驼峰形式字符串 '''
    # 这里re.sub()函数第二个替换参数用到了一个匿名回调函数，回调函数的参数x为一个匹配对象，返回值为一个处理后的字符串
    # lambda x: x.group(1)[1].upper()
    sub = re.sub(r'(_\w)|(^\w)', to_up, underline_str)
    return sub


def to_small_camel_name(underline_str):
    ''' 下划线形式字符串转成小驼峰形式 :param underline_str: 下划线形式字符串 :return: 驼峰形式字符串 '''
    # 这里re.sub()函数第二个替换参数用到了一个匿名回调函数，回调函数的参数x为一个匹配对象，返回值为一个处理后的字符串
    # lambda x: x.group(1)[1].upper()
    sub = re.sub(r'(_\w)', to_up, underline_str)
    return sub




from .Dict import Dict


def check(item):
    if not type(item) == dict:
        raise TypeError('element require dict type')


def generator_check(g):
    for item in g:
        check(item)
        yield item


class DictList(list):
    def __init__(self, *args, **kwargs):
        if len(args):
            args = (generator_check(args[0]),)

        super().__init__(*args, **kwargs)

    def __getitem__(self, index):
        return super().__getitem__(index)

    def __setitem__(self, key, item):
        check(item)
        return super().__setitem__(key, item)

    def append(self, item):
        check(item)
        super().append(item)

    def insert(self, item):
        check(item)
        super().append(item)



import wpath
from torm import Model
from torm import f


class Group(Model):
    __config__ = "mongo"

    group = f.EmailList()
    group_hash = f.Str()
    display_name = f.Str()

    update_at = f.Timestamp()
    create_at = f.Timestamp()


user_email = 'thhk06@163.com'
groups = Group.FindMany({"group": user_email})
print(groups)



import decimal
import json


class Dict(dict):
    def __init__(self, *args, **kws):
        d = args[0]
        if d is not None:
            for k, v in d.items():
                self[k] = str(v) if isinstance(v, decimal.Decimal) else v
        return super().__init__()

    def __key(self, key):
        return "" if key is None else key.lower()

    def __setattr__(self, key, value):
        self[self.__key(key)] = value

    def __getattr__(self, key):
        return self.get(self.__key(key))

    def __getitem__(self, key):
        return super().get(self.__key(key))

    def __setitem__(self, key, value):
        return super().__setitem__(self.__key(key), value)

    def to_json(self):
        return json.dumps(self)

    def from_json(self, json_str):
        return json.loads(json_str)



from torm.connection import MongoConnection, MysqlConnection
from combomethod import combomethod


def _connection(config):
    if config['db_type'] == 'mongo':
        return MongoConnection(config)

    if config['db_type'] == 'mysql':
        return MysqlConnection(config)


class BaseBuilder():

    def __new__(cls, *args, **kws):
        return super().__new__(cls)

    def __init__(self, *args, **kwargs):
        super().__init__()

    @combomethod
    def validate_type(self, item):
        if isinstance(item, self.__class__):
            return True
        else:
            try:
                item = self(item)
            except Exception as e:
                raise TypeError(
                    f'item must be or change to {self.__class__} type')
            return True

    def _connection(self):
        config = self.__config__

        if config['db_type'] == 'mongo':
            return MongoConnection(config)

        if config['db_type'] == 'mysql':
            return MysqlConnection(config)



class Field(object):

    def __init__(self, *args, **kws):
        self.value = kws.get('default')  # 当前值
        self.default = kws.get('default')  # 默认值
        self.field_type = kws.get('field_type')  # 数据类型

        self.key = kws.get('key')  # 字段是否是key键

        self.left = kws.get('left')  # 最小值
        self.right = kws.get('right')  # 最大值
        self.meta = kws.get('meta')  # 边界判断规则

        self.only_db_types = kws.get('only_db_types', [])

    def __str__(self):
        return f'<{self.__class__.__name__} value={self.value}>'

    def __delete__(self, instance):
        raise AttributeError(r"%s attribute '%s' is forbidden to be deleted." % (
            instance.__class__.__name__, self.name))

    def __set_name__(self, owner, name):
        self.model = owner.__name__
        self.name = name



from torm.field import Field
from torm.utl.Error import *


class Timestamp(Field):

    def __init__(self, *args, **kws):

        default = {
            'left': None,
            'right': None,
            'meta': 'left',
            'default': 0,
            'field_type': 'int(16)',
            'key': False,
            'only_db_types': None
        }

        default.update(kws)

        super().__init__(**default)

    def __get__(self, instance, owner):
        return self.value

    def __set__(self, instance, value):
        if self.validate(value):
            self.value = value

    def validate(self, value):
        model = self.model
        key = self.name
        if type(value) != int:
            raise error_type(key, value, model, int)
        if value > 9999999999:  # 支持的最大时间戳
            raise ValueError(
                f"{key} value {value} must smaller than {model} support max value 9999999999.")

        return True



import wpath
from torm import Model
from torm import f


class User(Model):
    __config__ = "mysql"

    id = f.Int(key=True)
    display_name = f.Str(32)
    name = f.Str(32)
    role = f.Str(64, default="user")

    phone = f.Str(32)
    email = f.Email()
    password = f.Str(64)

    register_finish_step = f.Int(default=1)
    verify_code = f.Str(6)
    verify_code_timestamp = f.Int()
    current_address = f.Str(42)

    login_timestamp = f.Int()
    update_at = f.Int()
    create_at = f.Int()  # 创建时间


def test_create_table():
    User.create_table()


def test_insert_one():
    user = User(name='kai', display_name='Kai', phone=None, email=None)
    print(user)
    u = User.InsertOne(user)
    # print(u)


def test_find_one():
    user = User.FindOne()
    print(user)
    # user.name = 'Kai'
    # User.UpdateOne({'id': user.id}, user)


def test_find_many():
    r = User.FindMany({'name': 'user_name'})
    print(r)


def test_update_one():
    r = User.UpdateOne({'name': 'user_name'}, {'display_name': 'User Name'})
    print(r)


def test_delete_one():
    r = User.DeleteOne({'name': 'user_name'})
    print(r)


def test_like():
    keyword = 'thhk'
    limit = 2
    users = User.where('email', 'like', "%" + keyword +
                       "%").limit(limit).get()
    print(users)


if __name__ == "__main__":
    test_find_one()
    # test_insert_one()
    # test_update_one()
    # test_find_one()
    # test_delete_one()
    # test_like()



from torm.builder.BaseBuilder import BaseBuilder
from pymysql.cursors import DictCursor

from torm.utl.Expression import expression as expr, Expression
from combomethod import combomethod


import inspect

from torm.utl.Map import Map


def get_var_name(var, _depth=3):
    # 当前命名空间
    fr = inspect.currentframe()
    for i in range(_depth):
        fr = fr.f_back  # f_back获取父空间
    vars = fr.f_locals.items()  # 获取该命名空间的所有变量
    for var_name, var_val in vars:
        if var_val is var:  # 如果该变量的和var相同，则var_name就是var的变量名
            return var_name
    return None


def parse_args(args, kwargs, _depth=3):
    where = {}

    for arg in args:
        if type(arg) in [dict, set, Map]:
            arg = Map(arg, _depth=_depth)
            where.update(arg)
        elif type(arg) == str:
            key = get_var_name(arg, _depth=_depth)
            where[key] = arg

    where.update(kwargs)
    return where


class MysqlBuilder(BaseBuilder):
    operators = [
        '=', '<', '>', '<=', '>=', '<>', '!=',
        'like', 'like binary', 'not like', 'between', 'ilike',
        '&', '|', '^', '<<', '>>',
        'rlike', 'regexp', 'not regexp',
        '~', '~*', '!~', '!~*', 'similar to',
        'not similar to', 'not ilike', '~~*', '!~~*', 'in', 'not in', 'not between'
    ]

    __select__ = []                            # 检索的字段
    __where__ = []
    __orwhere__ = []                           # orwhere处理逻辑
    __whereor__ = []                           # orwhere处理逻辑

    __offset__ = None                          # offset
    __limit__ = None                           # 检索的数据条数
    __orderby__ = []                           # 排序字段
    __groupby__ = []  # 排序字段

    __lock__ = None                            # lock
    __join__ = []                              # leftjoin
    __union__ = []                             # union & unionall
    __on__ = []                                # leftjoin

    __having__ = None                          # having
    __subquery__ = []                          # subquery

    def __init__(self, *args, **kwargs):
        super().__init__()

    @combomethod
    def reset(self):
        self.__select__ = []
        self.__where__ = []
        self.__orwhere__ = []
        self.__whereor__ = []

        self.__offset__ = None
        self.__limit__ = None
        self.__orderby__ = []
        self.__groupby__ = []

        self.__lock__ = None
        self.__join__ = []
        self.__union__ = []
        self.__on__ = []

        self.__having__ = None
        self.__subquery__ = []

    # 增

    def create(self):
        data = self.to_dict()

        # 如果数据库是mysql，默认id无效
        if 'id' in data:
            data.pop('id')

        if data:
            if data and isinstance(data, dict):
                data = [
                    {key: value for key, value in data.items() if key in self.__field__}]

            sql = self._compile_create(data)
            self.connection.execute(sql)
        return self.lastid()

    def insert(self, columns, data):
        self.connection.execute(self._compile_insert(columns, data))
        return self

    @combomethod
    def create_table(self):
        charset = self.config['charset']
        key = "id"

        field_sql = ""
        for k in self.__field__:
            v = self.__fields__[k]
            if v.__class__.__name__ == 'Str':
                field_sql += f"`{v.name}` {v.field_type},"
            else:
                field_sql += f"`{v.name}` {v.field_type} NOT NULL DEFAULT {v.default},"

        sql = f"CREATE TABLE `{self.table_name}` ( {field_sql} PRIMARY KEY (`{key}`) ) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT TORM_CHARSET={charset}; "
        self.connection.execute(sql)
        return None

    # 删
    def delete(self):
        return self.connection.execute(self._compile_delete())

    # 改
    def update(self, data):
        if data and isinstance(data, dict):
            data = {key: value for key,
                    value in data.items() if key in self.__field__}
            sql = self._compile_update(data)
            print(sql)
            return self.connection.execute(sql)

    @combomethod
    def first(self):
        self.__limit__ = 1
        data = self.get()
        if data:
            return data.pop()
        return data

    # 查
    @combomethod
    def get(self):
        sql = self._compile_select()
        try:
            result = self.connection.execute(sql, DictCursor)
        except Exception as e:
            print(sql)
            raise e
        return [Map(index) for index in result]

    @combomethod
    def all(self):
        sql = self._compile_select()
        try:
            result = self.connection.execute(sql, DictCursor)
        except Exception as e:
            print(sql)
            raise e
        return [Map(index) for index in result]

    # 计数
    @combomethod
    def count(self):
        self.__select__ = ['count(*) as aggregate']
        data = self.first()
        return data['aggregate'] if data else None

    # 聚合
    @combomethod
    def groupby(self, *args):
        self.__groupby__ = self._format_columns(list(args))
        return self

    # 常见sql操作
    @combomethod
    def select(self, *args):
        self.__select__ = self._format_columns(list(args))
        return self

    @combomethod
    def where(self, *args):
        length = args.__len__()
        if length == 1 and isinstance(args[0], dict):
            if args[0]:
                self.__where__.append(args[0])
        elif length == 2:
            self.__where__.append(
                {args[0]: self._check_columns_value(args[1])})
        elif length == 3:
            if args[1] in self.operators:
                if args[1] == '=':
                    self.__where__.append(
                        {args[0]: self._check_columns_value(args[2])})
                else:
                    self.__where__.append(
                        (args[0], args[1], self._check_columns_value(args[2])))
            else:
                raise Exception(
                    'operator key world not found: "{}"'.format(args[1]))
        else:
            raise Exception('bad parameters in where function')
        return self

    def lastid(self):
        data = self.connection.execute(self._compile_lastid())
        return data[0][0] if data and data[0] and data[0][0] else None

    def _compile_lastid(self):
        return 'select last_insert_id() as lastid'

    @combomethod
    def orderby(self, column, direction='asc'):
        if direction.lower() == 'asc':
            self.__orderby__.append(expr.format_column(column, self))
        else:
            self.__orderby__.append(expr.format_column(
                column, self.__model__) + ' desc')
        return self

    @combomethod
    def offset(self, number):
        if number < 0:
            raise Exception('offset number invalid')
        self.__offset__ = int(number)
        return self

    @combomethod
    def limit(self, number):
        if number <= 0:
            raise Exception('take number invalid')
        self.__limit__ = int(number)
        return self

    @combomethod
    def _columnize(self, columns):
        return tuple(columns).__str__().replace('\'', '`')

    @combomethod
    def _format_columns(self, columns):
        return [expr.format_column(column, self) for column in columns]

    @combomethod
    def _check_columns_value(self, value):
        if self.__subquery__ and len(self.__subquery__) >= 2 and isinstance(value, str):
            tmp = value.split('.')
            if len(tmp) == 2 and tmp[0] in self._get_subquery_alias():
                return Expression(value)
        return value

    @combomethod
    def _compile_create(self, data):
        return "insert into {} {} values {}".format(self.table_name, self._columnize(data[0]), self._valueize(data))

    @combomethod
    def _compile_select(self):

        if len(self.__select__) == 0:
            self.__select__.append('*')

        sub_sql = ''.join(
            [self._compile_where(), self._compile_whereor(), self._compile_orwhere(), self._compile_groupby(), self._compile_orderby(),
             self._compile_having(), self._compile_limit(), self._compile_offset(), self._compile_lock()])

        join_sql = ''.join(self._compile_leftjoin())

        union_sql = ''.join(self._compile_union())

        return_sql = "select {} from {}{}{}".format(
            ','.join(self.__select__), self.table_name, join_sql, sub_sql, union_sql)

        return return_sql

    @combomethod
    def _compile_where(self):
        if len(self.__where__) > 0:
            sqlstr = []
            for index in self.__where__:
                if isinstance(index, dict):
                    sqlstr.append(' and '.join(self._compile_dict(index)))
                elif isinstance(index, tuple):
                    sqlstr.append(self._compile_tuple(index))
            return ' where {}'.format(' and '.join(sqlstr))
        return ''

    @combomethod
    def _compile_orwhere(self):
        if len(self.__orwhere__) > 0:
            sqlstr = []
            for index in self.__orwhere__:
                if isinstance(index, dict):
                    subsql = self._compile_dict(index)
                    if len(subsql) == 1:
                        sqlstr.append(subsql.pop())
                    else:
                        sqlstr.append('({})'.format(' and '.join(subsql)))
                elif isinstance(index, tuple):
                    sqlstr.append(self._compile_tuple(index))
                elif isinstance(index, list):
                    subsql = []
                    for items in index:
                        if len(items) == 2:
                            subsql.append(
                                self._compile_keyvalue(items[0], items[1]))
                        if len(items) == 3:
                            subsql.append(self._compile_tuple(
                                (items[0], items[1], items[2])))
                    sqlstr.append('({})'.format(' and '.join(subsql)))
                else:
                    raise Exception(
                        'undefined query condition {}'.format(index.__str__()))
            if len(self.__where__) > 0:
                return ' or {}'.format(' or '.join(sqlstr))
            return ' where {}'.format(' or '.join(sqlstr))
        return ''

    @combomethod
    def _compile_whereor(self):
        if len(self.__whereor__) > 0:
            sqlstr = []
            for index in self.__whereor__:
                subsql = []
                for item in index:
                    if isinstance(item, dict):
                        if len(item) == 1:
                            subsql.append(self._compile_dict(item).pop())
                        else:
                            subsql.append(
                                '(' + ' and '.join(self._compile_dict(item)) + ')')
                    elif isinstance(item, list):
                        if isinstance(item[0], str):
                            subsql.append(self._compile_tuple(tuple(item)))
                        else:
                            subsql.append(self._compile_lists(item))
                    elif isinstance(item, tuple):
                        subsql.append(self._compile_tuple(item))
                    else:
                        raise Exception('whereor param invalid')
                sqlstr.append(' or '.join(subsql))
            if len(self.__where__) > 0:
                return ' and ({})'.format(' or '.join(sqlstr))
            return ' where ({})'.format(' or '.join(sqlstr))
        return ''

    @combomethod
    def _compile_groupby(self):
        return '' if len(self.__groupby__) == 0 else ' group by ' + ','.join(self.__groupby__)

    @combomethod
    def _compile_orderby(self):
        return '' if len(self.__orderby__) == 0 else ' order by ' + ','.join(self.__orderby__)

    @combomethod
    def _compile_limit(self):
        return '' if self.__limit__ is None else ' limit {}'.format(self.__limit__)

    @combomethod
    def _compile_offset(self):
        return '' if self.__offset__ is None else ' offset {}'.format(self.__offset__)

    @combomethod
    def _compile_having(self):
        if self.__having__:
            return self.__having__
        return ''

    @combomethod
    def _compile_lock(self):
        return '' if self.__lock__ is None else self.__lock__

    @combomethod
    def _compile_leftjoin(self):
        if self.__join__:
            return ' ' + ' '.join(['{} {} on {}'.format(index, value._tablename(), value._compile_on()) for (index, value) in
                                   self.__join__])
        return ''

    @combomethod
    def _compile_union(self):
        if self.__union__:
            return ' ' + ' '.join(['{} ({})'.format(index, value.tosql()) for (index, value) in self.__union__])
        return ''

    @combomethod
    def _compile_dict(self, data):

        return ['{}={}'.format(expr.format_column(index, self), expr.format_string(value)).replace("'NULL'", 'null').replace("None", 'null') for index, value in data.items()]

    @combomethod
    def _compile_tuple(self, data):
        if data[1] in ['in', 'not in']:
            return self._compile_in((data[0], data[1], data[2]))
        elif data[1] in ['between', 'not between']:
            return self._compile_between((data[0], data[1], data[2]))
        return '{} {} {}'.format(expr.format_column(data[0], self), data[1], expr.format_string(data[2]))

    @combomethod
    def _compile_update(self, data):
        return "update {} set {}{}".format(self.table_name, ','.join(self._compile_dict(data)), self._compile_where())

    @combomethod
    def _compile_delete(self):
        return 'delete from {}{}'.format(self.table_name, self._compile_where())

    @combomethod
    def _valueize(self, data):
        # return ','.join([tuple(index.values()).__str__() for index in data])
        return ','.join([str(tuple(index.values())).replace("'NULL'", 'null').replace("None", 'null') for index in data])

    @combomethod
    def InsertOne(self, item):
        self.reset()
        self.validate_type(item)
        item = self(item)
        return item.create()

    @combomethod
    def InsertMany(self, items):
        self.reset()
        all_validate_type = all([self.validate_type(item) for item in items])
        if not all_validate_type:
            raise TypeError(f'all list element must be {self.__class__} type')

        result = []
        for item in items:
            item = self(item)
            result.append(item.create())
        return result

    @combomethod
    def FindOne(self, *args, **kwargs):
        self.reset()
        where = parse_args(args, kwargs, _depth=4)
        items = self.where(where).limit(1).get()
        if not items:
            return None
        item = Map(items[0])
        return item

    @combomethod
    def FindMany(self, *args, **kwargs):
        self.reset()
        where = parse_args(args, kwargs, _depth=4)
        items = self.where(where).get()
        if not items:
            return None
        items = [Map(item) for item in items]
        return items

    @combomethod
    def UpdateOne(self, where={}, item={}):
        self.reset()

        if not where:
            return None
        if not item:
            return None
        where = Map(where, _depth=4)

        if isinstance(item, self.__class__):
            item = item.to_dict()

        # 禁止更新id字段
        if "id" in item:
            item.pop("id")

        # 先查找第一个，然后更新
        _item = self.FindOne(where)
        if _item:
            return self.where({'id': _item.id}).update(item)
        else:
            return None

    @combomethod
    def DeleteOne(self, *args, **kwargs):
        self.reset()
        where = parse_args(args, kwargs, _depth=4)
        _item = self.FindOne(where)
        if _item:
            return self.where({'id': _item.id}).delete()
        else:
            return None



from torm.connection.ConnectionMetaClass import *


class Connection(metaclass=ConnectionMetaclass):
    __config__ = ""

    def __init__(self, config):
        self.__config__ = config
        self.db_name = config['db']



class BaseError(Exception):
    def __init__(self, msg):
        self.msg = msg

    def __bool__(self):
        return False


class TypeError(BaseError, TypeError):
    pass


class ValueError(BaseError, ValueError):
    pass


class ValidateError(BaseError):
    pass


class ConnectionError(BaseError):
    pass


def error_type(key, value, model, _type):
    if type(value) == str:
        value = '\'%s\'' % value
    return TypeError(
        'invalid type of %s, %s.%s value must be %s type.' % (value, model, key, _type))


def error_gt(key, value, model, limit):
    return ValueError(
        'invalid %s=%s, %s.%s must great than or equal to %s.' % (key, value, model, key, limit))


def error_g(key, value, model, limit):
    return ValueError(
        'invalid %s=%s, %s.%s must great than %s.' % (key, value, model, key, limit))


def error_lt(key, value, model, limit):
    return ValueError(
        'invalid %s=%s, %s.%s must smaller than or equal to %s.' % (key, value, model, key, limit))


def error_l(key, value, model, limit):
    return ValueError(
        'invalid %s=%s, %s.%s value must smaller than %s.' % (key, value, model, key, limit))



from torm.field import Field
from torm.utl.Error import *

import re


class Email(Field):

    def __init__(self, *args, **kws):
        default = {
            'short': None,
            'long': None,
            'meta': 'short',
            'default': None,
            'field_type': 'email',
            'key': False,
            'only_db_types': None
        }
        default.update(kws)
        super().__init__(**default)

    def __get__(self, instance, owner):
        return self.value

    def __set__(self, instance, value):
        if self.validate(value):
            self.value = value

    def validate(self, value):
        model = self.model
        key = self.name
        if value == "" or value == None:
            return True

        if type(value) != str:
            raise error_type(key, value, model, str)
        p = re.compile(r'''(
            [a-zA-Z0-9._%+-]+           # email-username
            @
            [a-zA-Z0-9.-]+              # domain-name
            \.[a-zA-Z]+             # dot-something
            )''', re.VERBOSE)

        s = re.search(p, value)
        if s:
            position = s.span()         # 第一个匹配的起始位置
            if len(value) == position[1]:
                return True
        raise error_type(key, value, model, "email")



from .Connection import Connection
from .Pool.MysqlConnectionPool import connectionPool


class MysqlConnection(Connection):
    _connection = {}

    _instance = None
    # 单例模式

    @classmethod
    def __new__(cls, *args, **kws):
        if not cls._instance:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, config):
        super().__init__(config)
        self._config = config

    def connect(self):
        db = self._config['db']

        if self._connection.get(db, None) is None:
            self._connection[db] = connectionPool.connection(self._config, db)
        return self._connection[db]

    def execute(self, sql, cursorclass=None):
        if cursorclass:
            cursor = self.connect().cursor(cursor=cursorclass)
        else:
            cursor = self.connect().cursor()
        error = None
        try:
            cursor.execute(sql)
        except Exception as e:
            e = str(e).strip("(").strip(")")
            error = Exception(e)
        if error:
            raise error

        data = cursor.fetchall()

        if not hasattr(self, '_transaction'):
            self.connect().commit()
        cursor.close()
        return data

    def transaction(self, callback):
        try:
            self.start()
            result = callback()
            self.connect().commit()
            self.end()
            return result
        except Exception as e:
            self.connect().rollback()
            self.end()
            raise e

    def transaction_wrapper(self, callback):
        @wraps(callback)
        def wrapper(*args, **kwargs):
            try:
                self.start()
                result = callback(*args, **kwargs)
                self.connect().commit()
                self.end()
                return result
            except Exception as e:
                self.connect().rollback()
                self.end()
                raise e

        return wrapper

    @classmethod
    def instance(cls, database, config):
        if cls._instance.get(database, None) is None:
            cls._instance[database] = MysqlConnection(database, config)
        return cls._instance.get(database, None)

    def start(self):
        self._transaction = True

    def end(self):
        del self._transaction

    def _sql_create_table(self):
        pass



from torm.field.Field import Field
# 基础数据类型
from torm.field.Int import Int
from torm.field.Float import Float
from torm.field.Str import Str
from torm.field.Bool import Bool

# 语义化数据类型
from torm.field.Email import Email
from torm.field.Timestamp import Timestamp

# 复杂数据类型，只有mongodb支持
from torm.field.Dict import Dict
from torm.field.Map import Map

# 复杂列表数据类型，只有mongodb支持
from torm.field.List import List
from torm.field.EmailList import EmailList
from torm.field.DictList import DictList




from .Connection import Connection
import pymongo
from urllib import parse


class MongoConnection(Connection):
    _instance = None
    _conn = None
    _config = None

    # 单例模式
    @classmethod
    def __new__(cls, *args, **kws):
        if not cls._instance:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, config):
        super().__init__(config)
        self._config = config

    # 连接数据库
    def connect(self):
        config = self._config
        if config['url']:
            MONGO_URL = config['url']
        elif 'user' and 'password' in config:
            user = parse.quote_plus(config['user'])
            password = parse.quote_plus(config['password'])
            MONGO_URL = f"mongodb://{user}:{password}@{config['host']}:{config['port']}/admin"
        else:
            MONGO_URL = f"mongodb://{config['host']}:{config['port']}/admin"

        conn = pymongo.MongoClient(MONGO_URL)

        return conn[config['db']]

    # 返回数据库
    def db(self, tablename):
        return self.connect()[tablename]

    def table(self, builder):
        tb = builder.table_name
        return self.db(tb)
    # 增

    def create(self, builder, data):
        tb = builder.table_name
        return self.db(tb).insert(data)

    # 删
    def delete(self, builder):
        return self.db(builder.table_name).delete_many(builder.__where__)

    # 改
    def update(self, builder, data, **kwargs):
        return self.db(builder.table_name).update_many(builder.__where__, data, **kwargs)

    # 查
    def get(self, builder):

        _select = builder.__select__ if builder.__select__ else {'_id': 0}

        tb = builder.table_name

        model = self.db(tb).find(builder.__where__, _select).skip(
            builder.__offset__).limit(builder.__limit__)

        return list(model)

    # 计数
    def count(self, builder):
        return self.db(builder.table_name).find(builder.__where__).count()

    # 聚合
    def groupby(self, builder):
        # 聚合操作的列表
        aggre = [builder.where_to_match(), builder.__groupby__]
        if builder.__orderby__.__len__() > 0:
            aggre.append(builder._compile_aggregate_orderby())
        if builder.__offset__ > 0:
            aggre.append(builder._compile_aggregate_offset())
        if builder.__limit__ > 0:
            aggre.append(builder._compile_aggregate_limit())
        aggre.append(builder._compile_aggregate_project())

        # mongo的聚合
        return self.db(builder.table_name).aggregate(aggre)



import wpath
from torm import Model
from torm import f
from torm.utl.Map import Map
import json


class Record(Model):
    __config__ = "mongo"

    id = f.Str()
    hash = f.Str()
    subject = f.Str()
    promoter = f.Str()
    recorder = f.Email()

    attachments_num = f.Int()
    mail_id = f.Str()
    group_hash = f.Str()

    status = f.Str()
    tx_status = f.Str()

    commits = f.Dict()
    group = f.EmailList()
    hashtags = f.Dict  # f.Map()

    create_at = f.Int()
    update_at = f.Int()


def test_insert_one():
    commits = {'user_id': 'some thing'}

    hashtags = {'1': ['']}
    record = Record({
        "hash": '12',
        "recorder": None,
        "subject": "test",
        "hashtags": hashtags,
        "group": ["thhk06@163.com", "kai.he@taraxa.io"],
        "commits": commits
    })

    # for k in record:
    #     print(k)
    # a = Map(record)
    # print(a)

    # for i in record:
    #     print(record[i])

    # record = {
    #     "id": '5eacf4e81e57fa47f0a732c4',
    #     "hash": '12',
    #     "recorder": "thhk06@163.com",
    #     "subject": "test",
    #     "hashtags": hashtags,
    #     "group": ["thhk06@163.com", "kai.he@taraxa.io"],
    #     "commits": commits
    # }

    r = Record.InsertOne(record)
    print(r)


def test_json():
    commits = {'user_id': 'some thing'}

    hashtags = {'1': ['']}
    record = Record({
        "hash": '12',
        "recorder": "thhk06@163.com",
        "subject": "test",
        "hashtags": hashtags,
        "group": ["thhk06@163.com", "kai.he@taraxa.io"],
        "commits": commits
    })
    r = json.dumps(record)
    print(r)


def test_find_one():
    # record = Record()
    # record.FindOne()
    record = Record.FindOne()
    print(record)


def test_find_many():
    records = Record.FindMany()
    print(records)


def test_update_one():
    r = Record.UpdateOne(hash, {'subject': 'test5'})
    print(r)


if __name__ == "__main__":
    test_find_one()

    # test_insert_one()

    # test_find()

    # print(r)
    # print(Record.__dict__.keys())

    # r = Record.where({'id': '5ead0d7f34e0b4dddeb234e3'}).get()
    #   r = Record.where('id', '=', '5ead0d7f34e0b4dddeb234e3').first()
    # print(r)
    # test_find_many()

    # test_find()

    # test_json()



import wpath
from torm import Model
from torm import f
import time


class Account(Model):
    __config__ = 'mysql'

    id = f.Int()
    user_id = f.Int()
    is_current = f.Bool()
    address = f.Str(42)
    private_key = f.Str(128)

    update_at = f.Timestamp()
    create_at = f.Timestamp()


privateKey = "0x7b2b3db28278e11df1648ba8c741e9d0d67c7794809124afbfbee6f1e505f5be"
address = "0xeCaA62E08a7760C34229D4B5B0FA37B89A264a23"
secretKey = "123456"
private_key = "U2FsdGVkX1/6EIiEKeTvbEkgFxseH0dbfafvMtlNGxdwq03PESAK6jmEUGsojEWp6Pkl0SWkrdG3zUbb29k+BlcSNDmAVoSm904mWo5oGmhQpk5MuQ/zxBafsEPZMLfr"


t = int(time.time())

account = Account(
    user_id=13,
    is_current=True,
    address=address,
    private_key=private_key,
    update_at=t,
    create_at=t)

print(account)

r = Account.InsertOne(account)
print(r)



from collections.abc import Iterable
import inspect


def get_var_name(var, depth=2):
    # 当前命名空间
    fr = inspect.currentframe()
    for i in range(depth):
        fr = fr.f_back  # f_back获取父空间
    vars = fr.f_locals.items()  # 获取该命名空间的所有变量
    for var_name, var_val in vars:
        if var_val is var:  # 如果该变量的和var相同，则var_name就是var的变量名
            return var_name
    return None


class Map(dict):
    ###
    # js风格的dict,可以用["key"]或.key访问元素
    # python的dict,原有的属性除了魔法属性，还有：
    # 'clear', 'copy', 'fromkeys', 'get', 'items', 'keys',
    # 'pop', 'popitem', 'setdefault', 'update', 'values'
    # Map应该避免用.key访问以上同名的元素，这种情况可以用["key"]的方式访问
    ###

    def __init__(self, *args, _depth=2, **kwargs):
        for arg in args:
            if arg == None:
                continue
            if type(arg) == str:
                self[get_var_name(arg, _depth)] = arg
            elif type(arg) == set:
                for e in arg:
                    self[get_var_name(e, _depth)] = e
            elif isinstance(arg, Iterable):
                super().__init__(arg)
            else:
                self[get_var_name(arg, _depth)] = arg
        super().__init__(kwargs)

    def origin_method(self):
        return ['clear', 'copy', 'fromkeys', 'get', 'items', 'keys', 'pop', 'popitem', 'setdefault', 'update', 'values']

    def __getattribute__(self, name):
        try:
            # 原有字典有该属性时，直接返回属性
            return super().__getattribute__(name)
        except AttributeError:
            try:  # 原有字典没有该属性时，尝试查找字典的key
                return self[name]
            except KeyError:  # 如果还找不到，返回None
                return None

    def __setattr__(self, name, value):
        if name == "origin_method":
            raise KeyError(f'Should not overlap Map origin method "{name}"')
        if name in self.origin_method():
            raise KeyError(f'Should not overlap Map origin method "{name}"')
        try:  # 原有字典有该属性时，将值绑定到该属性
            _value = super().__getattribute__(name)
            super().__setattr__(name, value)
        except AttributeError:  # 原有字典没有该属性时，将该属性作为字典的key，将值存入字典
            self[name] = value

    def pretty(self):
        values = ', '.join('\n    {} = {!r}'.format(
            i, self[i]) for i in self.keys())
        return '{}({}\n)'.format(self.__class__.__name__, values)


if __name__ == "__main__":
    a = {'a': 3, 'b': {'c': 4}}
    m = Map(a)
    print(m.pretty())



from torm.field import Field
from torm.utl.Error import *


class Bool(Field):

    def __init__(self, *args, **kws):
        default = {
            'short': None,
            'long': None,
            'meta': 'short',
            'default': False,
            'field_type': 'bool',
            'key': False,
            'only_db_types': None
        }
        super().__init__(**default)

    def __get__(self, instance, owner):
        return self.value

    def __set__(self, instance, value):
        if self.validate(value):
            self.value = value

    def validate(self, value):
        model = self.model
        key = self.name
        if type(value) != bool:
            raise error_type(key, value, model, bool)

        return True



import collections

from torm.utl.Config import config
from torm.utl.Utl import to_snake_name
from torm.field import *
from torm.builder import MongoBuilder, MysqlBuilder
from torm.connection import MongoConnection, MysqlConnection


def _connection(config):
    if config['db_type'] == 'mongo':
        return MongoConnection(config)
    if config['db_type'] == 'mysql':
        return MysqlConnection(config)


class ModelMetaclass(type):
    isinstance = False
    isclass = True

    @classmethod
    def __prepare__(cls, name, bases, **kws):
        return collections.OrderedDict()

    # 返回子类
    def __new__(cls, name, bases, attrs):
        '''
        name:类名
        bases:父类
        attrs:类的所有属性字典
        '''
        if name == 'Model':
            return type.__new__(cls, name, bases, attrs)  # 返回Model类型

        model_fields = dict()

        cls_keys = list(attrs.keys())
        for k in cls_keys:
            if isinstance(attrs[k], Field):
                model_fields[k] = attrs[k]

        field = model_fields.keys()
        attrs['__fields__'] = model_fields  # 保存字段的属性
        attrs['__field__'] = list(field)  # 保存字段名列表
        attrs['__config__'] = cls.init_config(cls, name, attrs)  # 连接配置

        attrs['config'] = attrs['__config__']
        attrs['db_name'] = attrs['__config__']['db']
        attrs['table_name'] = attrs['__config__']['table']
        attrs['connection'] = _connection(attrs['__config__'])

        # 根据数据库类型，继承各数据库的builder
        dbtype = attrs['__config__']['db_type']
        if dbtype == 'mongo':
            bases = bases + (MongoBuilder,)
        elif dbtype == 'mysql':
            bases = bases + (MysqlBuilder,)

        # 使Model同时继承builder
        return type.__new__(cls, name, bases, attrs)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def init_config(cls, name, attrs):
        __config = {}
        # 环境变量配置名称
        env_name = attrs.get("__config__", "default")

        db_type = attrs.get(
            "__dbtype__", config.env(env_name)('TORM_DB_TYPE'))

        # 配置名称
        __config['config_name'] = env_name
        # 数据库类型
        __config['db_type'] = db_type
        # 数据库名
        __config['db'] = attrs.get(
            "__dbname__",
            config.env(env_name)('TORM_DB')
        )
        # 表名
        __config['table'] = attrs.get(
            "__tablename__",
            to_snake_name(name)
        )

        # 数据库连接参数
        __config['url'] = config.env(env_name)('TORM_URL', default=None)
        __config['host'] = config.env(env_name)('TORM_HOST')
        __config['port'] = int(config.env(env_name)('TORM_PORT'))
        __config['charset'] = config.env(env_name)('TORM_CHARSET')

        # 数据库用户名和密码配置
        auth = config.env(env_name)("TORM_AUTH", default="off")
        if auth == "on":
            __config['user'] = config.env(env_name)('TORM_USER')
            __config['password'] = config.env(env_name)('TORM_PASSWORD')

        return __config

    def __getattribute__(self, key):
        if key == "__new__":
            return object.__new__(self)
        try:
            return object.__getattribute__(self, key)
        except:
            return getattr(object.__new__(self), key)



import pandas as pd
import time

from torm.model.ModelMetaclass import *
from collections import OrderedDict
from combomethod import combomethod
from torm.utl.Map import Map


class Model(metaclass=ModelMetaclass):
    def __init__(self, *args, **kws):
        # 实例化的参数
        attrs = {}
        if args:
            if args[0] == None:
                pass
            elif isinstance(args[0], dict):
                attrs = dict(args[0])
            else:
                attrs = dict(zip(self.__field__, args))
        attrs.update(kws)

        # 将属性绑定到Model上,只绑定field的属性，多余的丢弃
        # print(self.config['db_type'])  # 数据库类型
        db_type = self.config['db_type']

        for key in self.__field__:
            # 检查数据库是否支持该字段的类型
            if self.__fields__[key].only_db_types:
                if not self.config['db_type'] in self.__fields__[key].only_db_types:
                    raise TypeError(
                        f'type of "{key}" {self.__fields__[key]} can only be used in db {self.__fields__[key].only_db_types}')

            if key in attrs:  # 如果对象的参数有该字段的值，就采用该值
                if attrs[key] == None:  # 如果该值为None,采用该字段的默认值
                    setattr(self, key, self.__fields__[key].default)
                else:
                    setattr(self, key, attrs[key])
            else:  # 没有提供该值，就采用默认值
                setattr(self, key, self.__fields__[key].default)

        # 父类Builder初始化
        super().__init__()
        self.isinstance = True
        self.isclass = False

    def __len__(self):
        return len(self.__field__)

    def __getattribute__(self, key):
        return object.__getattribute__(self, key)

    def __getattr__(self, key):
        raise AttributeError(r"%s has no attribute or method '%s'" % (
            self.__class__.__name__, key))

    def __setattr__(self, key, value):
        # 限制可以绑定的属性
        if key in self.__field__:
            object.__setattr__(self, key, value)
        elif key in ['connection', 'config', 'table_name', 'isinstance', 'isclass'] or (key.startswith('__') and key.endswith('__')):
            object.__setattr__(self, key, value)
        else:
            raise AttributeError(r"%s can not bind attribute '%s'" % (
                self.__class__.__name__, key))

    def __len__(self):
        return len(self.__field__)

    def __setitem__(self, key, value):
        if key in self.__field__:
            self.__setattr__(key, value)

    def __getitem__(self, key):
        if key in self.__field__:
            return self.__getattribute__(key)
        else:
            raise AttributeError(r"%s has no attribute '%s'" % (
                self.__class__.__name__, key))

    def __iter__(self):
        for key in self.__field__:
            yield key, self[key]

    def __str__(self, pretty=False):
        if not self.__bool__():
            return 'Null'
        values = ', '.join('{}={!r}'.format(
            i, self[i]) for i in self.__field__)
        return '{}({})'.format(self.__class__.__name__, values)

    def __bool__(self):
        # 如果所有字段都是空，则返回False,否则返回True
        bools = [bool(self[i]) for i in self.__field__]
        if True in bools:
            return True
        else:
            return False

    def __call__(self, *args, **kws):
        if len(args):
            if type(args[0]) == dict:
                kws.update(args[0])
        try:
            for k in self.__field__:
                v = kws.get(k)
                if v:
                    self.__setattr__(k, v)
            return self
        except Exception as e:
            return e

    def pretty(self):
        values = ', '.join('\n    {} = {!r}'.format(
            i, self[i]) for i in self.__field__)
        return '{}({}\n)'.format(self.__class__.__name__, values)

    def to_dict(self):
        d = dict()
        for i in self.__field__:
            d[i] = self[i]
        return d

    def to_ordict(self):
        d = OrderedDict()
        for i in self.__field__:
            d[i] = self[i]
        return d

    def to_map(self):
        return Map(self.to_ordict())

    @combomethod
    def to_df(self):
        return pd.DataFrame.from_records(self.get())

    def gets(self, id, name):
        print(id, name)



from torm.field import Field
from torm.utl.Error import *
import torm.utl


class MapList(Field):
    def __init__(self, *args, **kwargs):
        default = {
            'left': None,
            'right': None,
            'meta': 'left',
            'default': [],
            'field_type': 'list',
            'key': False,
            'only_db_types': ['mongo']
        }
        super().__init__(**default)

        # if len(args):
        #     args = (generator_check(args[0]),)

        # super().__init__(*args, **kwargs)

    def __get__(self, instance, owner):
        return self.value

    def __set__(self, instance, value):
        if self.validate(value):
            self.value = value

    def validate(self, value):
        model = self.model
        key = self.name
        if type(value) != list:
            raise error_type(key, value, model, list)
        self.generator_check(value)
        return True

    def check(self, item):
        if type(value) == torm.utl.Map.Map or type(value) == dict:
            raise TypeError('element require Map type')

    def generator_check(self, g):
        for item in g:
            self.check(item)
            yield item



from torm.field import Field
from torm.utl.Error import *
import torm.utl
import json


class Map(Field):
    def __init__(self, *args, **kws):

        default = {
            'left': None,
            'right': None,
            'meta': 'left',
            'default': torm.utl.Map.Map({}),
            'field_type': 'dict',
            'key': False,
            'only_db_types': None
        }
        super().__init__(**default)

    def __get__(self, instance, owner):
        return self.value

    def __set__(self, instance, value):
        if self.validate(value):
            self.value = value

    def validate(self, value):
        model = self.model
        key = self.name
        if type(value) == torm.utl.Map.Map or type(value) == dict:
            return True
        raise error_type(key, value, model, torm.utl.Map)

    def to_json(self):
        return json.dumps(self.value)

    def from_json(self, json_str):
        self.value = json.loads(json_str)
        return self




from torm.field import Field
from torm.utl.Error import *


class Float(Field):

    def __init__(self, *args, **kws):
        default = {
            'left': None,
            'right': None,
            'meta': 'left',
            'default': 0,
            'field_type': 'decimal(12,6)',
            'key': False,
            'only_db_types': None
        }
        default.update(self.boundary(args))
        default.update(kws)

        _meta = ['left', 'right', 'both', 'neither']
        if default['meta'] not in _meta:
            raise ValueError('meta must be in %s.' % _meta)

        super().__init__(**default)

    def __get__(self, instance, owner):

        return self.value

    def __set__(self, instance, value):
        if self.validate(value):
            self.value = value
        else:
            raise ValidateError()

    # 边界条件
    def boundary(self, args):
        meta = 'left'
        boundary = {}
        if len(args) == 1:
            if type(args[0]) == list:
                boundary = dict(zip(['left', 'right'], args[0]))
                meta = 'both'
            elif type(args[0]) == tuple:
                boundary = dict(zip(['left', 'right'], args[0]))
                meta = 'neither'
            elif type(args[0]) == str:
                _p = args[0]
                _p = _p.strip(" ")
                boundary_type = _p[0] + _p[-1]
                numbers = _p.strip("[").strip(
                    "(").strip(")").strip("]").split(',')
                boundary = dict(zip(['left', 'right'], numbers))
                _metas = {'[)': 'left', '(]': 'right',
                          '[]': 'both', '()': 'neither'}
                meta = _metas[boundary_type]

            else:
                if meta == 'left':
                    boundary = dict(zip(['left'], args))
                if meta == 'right':
                    boundary = dict(zip(['right'], args))

        if len(args) == 2:
            boundary = dict(zip(['left', 'right'], args))
        boundary['meta'] = meta

        return boundary

    # 检查数据
    def validate(self, value):
        model = self.model
        key = self.name

        if not type(value) in [int, float]:
            raise error_type(key, value, model, [int, float])

        if self.left:
            if self.meta in ['both', 'left']:
                if not value >= self.left:
                    raise error_gt(key, value, model, self.left)

            if self.meta in ['neither', 'right']:
                if not value > self.right:
                    raise error_g(key, value, model, self.left)
        if self.right:
            if self.meta in ['both', 'right']:
                if not value <= self.right:
                    raise error_lt(key, value, model, self.right)

            if self.meta in ['neither', 'left']:
                if not value < self.right:
                    raise error_l(key, value, model, self.right)



from setuptools import setup, find_namespace_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

NAME = "torm"

DESCRIPTION = "mongodb style python orm."

KEYWORDS = "mongodb orm"

AUTHOR = "Aeneas"

AUTHOR_EMAIL = "kai.he@taraxa.io"

URL = "https://github.com/cofepy"

VERSION = "0.1.6"

LICENSE = "MIT"

setup(
    name=NAME,
    version=VERSION,
    python_requires='>=3.7',
    install_requires=[],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
    ],
    keywords=KEYWORDS,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    license=LICENSE,
    packages=find_namespace_packages(),
    include_package_data=True,
    zip_safe=True,
)



from torm.field import Field
from torm.utl.Error import *
import json


class Dict(Field):
    def __init__(self, *args, **kws):
        default = {
            'left': None,
            'right': None,
            'meta': 'left',
            'default': {},
            'field_type': 'dict',
            'key': False,
            'only_db_types': ['mongo']
        }
        super().__init__(**default)

    def __get__(self, instance, owner):
        return self.value

    def __set__(self, instance, value):
        if self.validate(value):
            self.value = value

    def validate(self, value):
        model = self.model
        key = self.name
        if type(value) != dict:
            raise error_type(key, value, model, dict)
        return True

    def to_json(self):
        return json.dumps(self.value)

    def from_json(self, json_str):
        self.value = json.loads(json_str)
        return self



from torm.field import Field
from torm.utl.Error import *


class Str(Field):

    def __init__(self, *args, **kws):
        field_type = 'varchar(32)'
        if args:
            length = args[0]
            if type(length) != int:
                raise ValueError("Str length type must be int.")
            field_type = f'varchar({length})'

        default = {
            'short': None,
            'long': None,
            'meta': 'short',
            'default': None,
            'field_type': field_type,
            'key': False,
            'only_db_types': None
        }
        default.update(kws)
        super().__init__(**default)

    def __get__(self, instance, owner):
        return self.value

    def __set__(self, instance, value):
        if self.validate(value):
            self.value = value

    def validate(self, value):
        model = self.model
        key = self.name
        if value == None:
            return True

        if type(value) != str:
            raise error_type(key, value, model, str)

        return True
